@@ -20,7 +20,6 @@ from ibm_cloud_sdk_core import CP4DTokenManager
 from ibm_cloud_sdk_core import get_authenticator_from_environment
 from ibm_cloud_sdk_core.authenticators import (IAMAuthenticator, NoAuthAuthenticator, Authenticator,
                                                BasicAuthenticator, CloudPakForDataAuthenticator)
-from ibm_cloud_sdk_core.utils import strip_extra_slashes
 
 
 class IncludeExternalConfigService(BaseService):
@@ -134,7 +133,7 @@ def test_url_encoding():
     path0 = ' \"<>^`{}|/\\?#%[]'
     path0_encoded = '%20%22%3C%3E%5E%60%7B%7D%7C%2F%5C%3F%23%25%5B%5D'
     # All non-ASCII chars _must_ be encoded in path segments
-    path1 = u'比萨浇头'.encode('utf8')  # "pizza toppings"
+    path1 = '比萨浇头'.encode('utf8')  # "pizza toppings"
     path1_encoded = '%E6%AF%94%E8%90%A8%E6%B5%87%E5%A4%B4'
 
     path_encoded = '/v1/foo/' + path0_encoded + '/bar/' + path1_encoded + '/baz'
@@ -698,6 +697,7 @@ def test_user_agent_header():
     assert response.get_result().request.headers.__getitem__(
         'user-agent') == user_agent_header['User-Agent']
 
+
 @responses.activate
 def test_reserved_keys(caplog):
     service = AnyServiceV1('2021-07-02', authenticator=NoAuthAuthenticator())
@@ -723,6 +723,7 @@ def test_reserved_keys(caplog):
     assert caplog.record_tuples[2][2] == '"headers" has been removed from the request'
     assert caplog.record_tuples[3][2] == '"cookies" has been removed from the request'
 
+
 @responses.activate
 def test_ssl_error():
     responses.add(
@@ -741,7 +742,7 @@ def test_files_dict():
     form_data = {}
     with open(
         os.path.join(
-            os.path.dirname(__file__), '../resources/ibm-credentials-iam.env'), 'r') as file:
+            os.path.dirname(__file__), '../resources/ibm-credentials-iam.env'), 'r', encoding='utf-8') as file:
         form_data['file1'] = (None, file, 'application/octet-stream')
     form_data['string1'] = (None, 'hello', 'text/plain')
     request = service.prepare_request(
@@ -762,7 +763,7 @@ def test_files_list():
     form_data = []
     with open(
         os.path.join(
-            os.path.dirname(__file__), '../resources/ibm-credentials-iam.env'), 'r') as file:
+            os.path.dirname(__file__), '../resources/ibm-credentials-iam.env'), 'r', encoding='utf-8') as file:
         form_data.append(('file1', (None, file, 'application/octet-stream')))
     form_data.append(('string1', (None, 'hello', 'text/plain')))
     request = service.prepare_request(
@@ -783,17 +784,17 @@ def test_files_duplicate_parts():
     form_data = []
     with open(
         os.path.join(
-            os.path.dirname(__file__), '../resources/ibm-credentials-iam.env'), 'r') as file:
+            os.path.dirname(__file__), '../resources/ibm-credentials-iam.env'), 'r', encoding='utf-8') as file:
         form_data.append(
             ('creds_file', (None, file, 'application/octet-stream')))
     with open(
         os.path.join(
-            os.path.dirname(__file__), '../resources/ibm-credentials-basic.env'), 'r') as file:
+            os.path.dirname(__file__), '../resources/ibm-credentials-basic.env'), 'r', encoding='utf-8') as file:
         form_data.append(
             ('creds_file', (None, file, 'application/octet-stream')))
     with open(
         os.path.join(
-            os.path.dirname(__file__), '../resources/ibm-credentials-bearer.env'), 'r') as file:
+            os.path.dirname(__file__), '../resources/ibm-credentials-bearer.env'), 'r', encoding='utf-8') as file:
         form_data.append(
             ('creds_file', (None, file, 'application/octet-stream')))
     request = service.prepare_request(
@@ -810,56 +811,71 @@ def test_json():
     service = AnyServiceV1('2018-11-20', authenticator=NoAuthAuthenticator())
     req = service.prepare_request('POST', url='', headers={
                                   'X-opt-out': True}, data={'hello': 'world', 'fóó': 'bår'})
-    assert req.get('data') == b'{"hello": "world", "f\\u00f3\\u00f3": "b\\u00e5r"}'
+    assert req.get(
+        'data') == b'{"hello": "world", "f\\u00f3\\u00f3": "b\\u00e5r"}'
 
 
-def test_trailing_slash():
-    assert strip_extra_slashes('') == ''
-    assert strip_extra_slashes('//') == '/'
-    assert strip_extra_slashes('/////') == '/'
-    assert strip_extra_slashes('https://host') == 'https://host'
-    assert strip_extra_slashes('https://host/') == 'https://host/'
-    assert strip_extra_slashes('https://host//') == 'https://host/'
-    assert strip_extra_slashes('https://host/path') == 'https://host/path'
-    assert strip_extra_slashes('https://host/path/') == 'https://host/path/'
-    assert strip_extra_slashes('https://host/path//') == 'https://host/path/'
-    assert strip_extra_slashes('https://host//path//') == 'https://host//path/'
-
+def test_service_url_handling():
     service = AnyServiceV1(
-        '2018-11-20', service_url='https://host/', authenticator=NoAuthAuthenticator())
-    assert service.service_url == 'https://host/'
+        '2018-11-20', service_url='https://host///////', authenticator=NoAuthAuthenticator())
+    assert service.service_url == 'https://host'
+
     service.set_service_url('https://host/')
-    assert service.service_url == 'https://host/'
+    assert service.service_url == 'https://host'
+
     req = service.prepare_request('POST',
                                   url='/path/',
                                   headers={'X-opt-out': True},
                                   data={'hello': 'world'})
-    assert req.get('url') == 'https://host//path/'
+    assert req.get('url') == 'https://host/path/'
 
     service = AnyServiceV1(
         '2018-11-20', service_url='https://host/', authenticator=NoAuthAuthenticator())
-    assert service.service_url == 'https://host/'
+    assert service.service_url == 'https://host'
+
     service.set_service_url('https://host/')
-    assert service.service_url == 'https://host/'
+    assert service.service_url == 'https://host'
+
     req = service.prepare_request('POST',
                                   url='/',
                                   headers={'X-opt-out': True},
                                   data={'hello': 'world'})
     assert req.get('url') == 'https://host/'
 
+    req = service.prepare_request('POST',
+                                  url='////',
+                                  headers={'X-opt-out': True},
+                                  data={'hello': 'world'})
+    assert req.get('url') == 'https://host/'
+
     service.set_service_url(None)
     assert service.service_url is None
 
     service = AnyServiceV1('2018-11-20', service_url='/',
                            authenticator=NoAuthAuthenticator())
-    assert service.service_url == '/'
+    assert service.service_url == ''
+
     service.set_service_url('/')
-    assert service.service_url == '/'
-    req = service.prepare_request('POST',
-                                  url='/',
-                                  headers={'X-opt-out': True},
-                                  data={'hello': 'world'})
-    assert req.get('url') == '/'
+    assert service.service_url == ''
+
+    with pytest.raises(ValueError) as err:
+        service.prepare_request('POST',
+                                url='/',
+                                headers={'X-opt-out': True},
+                                data={'hello': 'world'})
+    assert str(err.value) == 'The service_url is required'
+
+
+def test_service_url_slash():
+    service = AnyServiceV1('2018-11-20', service_url='/',
+                           authenticator=NoAuthAuthenticator())
+    assert service.service_url == ''
+    with pytest.raises(ValueError) as err:
+        service.prepare_request('POST',
+                                url='/',
+                                headers={'X-opt-out': True},
+                                data={'hello': 'world'})
+    assert str(err.value) == 'The service_url is required'
 
 
 def test_service_url_not_set():
